﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.openMenuItem.Click     += openMenuItem_Click;
            this.saveAsMenuItem.Click   += saveAsMenuItem_Click;
            this.exitMenuItem.Click     += exitMenuItem_Click;
            this.cutoffScrollBar.Scroll += cutoffScrollBar_Scroll;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 열기 메뉴 항목 클릭시 처리하기 - openMenuItem_Click(sender, e)

        /// <summary>
        /// 열기 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void openMenuItem_Click(object sender, EventArgs e)
        {
            if(this.openFileDialog.ShowDialog() == DialogResult.OK)
            {
                this.sourcePictureBox.Image = GetBitmap(this.openFileDialog.FileName);

                ProcessBinaryContrast();
            }
        }

        #endregion
        #region 다른 이름으로 저장 메뉴 항목 클릭시 처리하기 - saveAsMenuItem_Click(sender, e)

        /// <summary>
        /// 다른 이름으로 저장 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void saveAsMenuItem_Click(object sender, EventArgs e)
        {
            if(this.saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                SaveBitmap((Bitmap)this.targetPictureBox.Image, this.saveFileDialog.FileName);
            }
        }

        #endregion
        #region 종료 메뉴 항목 클릭시 처리하기 - exitMenuItem_Click(sender, e)

        /// <summary>
        /// 종료 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void exitMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        #endregion
        #region 컷오프 스크롤바 스크롤시 처리하기 - cutoffScrollBar_Scroll(sender, e)

        /// <summary>
        /// 컷오프 스크롤바 스크롤시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void cutoffScrollBar_Scroll(object sender, ScrollEventArgs e)
        {
            this.cutoffValueLabel.Text = this.cutoffScrollBar.Value.ToString();

            ProcessBinaryContrast();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 비트맵 구하기 - GetBitmap(filePath)

        /// <summary>
        /// 비트맵 구하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        /// <returns>비트맵</returns>
        private Bitmap GetBitmap(string filePath)
        {
            using(Bitmap bitmap = new Bitmap(filePath))
            {
                return new Bitmap(bitmap);
            }
        }

        #endregion
        #region 이진 대비 적용하기 - ApplyBinaryContrast(targetBitmap, cutoff)

        /// <summary>
        /// 이진 대비 적용하기
        /// </summary>
        /// <param name="targetBitmap">타겟 비트맵</param>
        /// <param name="cutoff">컷오프</param>
        private void ApplyBinaryContrast(Bitmap targetBitmap, int cutoff)
        {
            BitmapHelper helper = new BitmapHelper(targetBitmap);

            helper.LockBitmap();

            for(int y = 0; y < targetBitmap.Height; y++)
            {
                for(int x = 0; x < targetBitmap.Width; x++)
                {
                    byte red;
                    byte green;
                    byte blue;
                    byte alpha;

                    helper.GetPixel(x, y, out red, out green, out blue, out alpha);

                    if(red + green + blue > cutoff)
                    {
                        helper.SetPixel(x, y, 255, 255, 255, 255);
                    }
                    else
                    {
                        helper.SetPixel(x, y, 0, 0, 0, 255);
                    }
                }
            }

            helper.UnlockBitmap();
        }

        #endregion
        #region 이진 대비 처리하기 - ProcessBinaryContrast()

        /// <summary>
        /// 이진 대비 처리하기
        /// </summary>
        private void ProcessBinaryContrast()
        {
            if(this.sourcePictureBox.Image == null)
            {
                return;
            }

            Cursor = Cursors.WaitCursor;

            Bitmap bitmap = new Bitmap(this.sourcePictureBox.Image);

            ApplyBinaryContrast(bitmap, 3 * this.cutoffScrollBar.Value);

            if(this.targetPictureBox.Image != null)
            {
                this.targetPictureBox.Image.Dispose();
            }

            this.targetPictureBox.Image   = bitmap;
            this.targetPictureBox.Left    = this.sourcePictureBox.Right + 4;

            Cursor = Cursors.Default;
        }

        #endregion
        #region 비트맵 저장하기 - SaveBitmap(bitmap, filePath)

        /// <summary>
        /// 비트맵 저장하기
        /// </summary>
        /// <param name="bitmap">비트맵</param>
        /// <param name="filePath">파일 경로</param>
        public void SaveBitmap(Bitmap bitmap, string filePath)
        {
            string extension = Path.GetExtension(filePath);

            switch(extension.ToLower())
            {
                case ".bmp"  : bitmap.Save(filePath, ImageFormat.Bmp ); break;
                case ".exif" : bitmap.Save(filePath, ImageFormat.Exif); break;
                case ".gif"  : bitmap.Save(filePath, ImageFormat.Gif ); break;
                case ".jpg"  :
                case ".jpeg" : bitmap.Save(filePath, ImageFormat.Jpeg); break;
                case ".png"  : bitmap.Save(filePath, ImageFormat.Png ); break;
                case ".tif"  :
                case ".tiff" : bitmap.Save(filePath, ImageFormat.Tiff); break;
                default      : throw new NotSupportedException("Unknown file extension " + extension);
            }
        }

        #endregion
    }
}